package com.landray.kmss.sys.attachment.watermark.util;

import java.util.Iterator;
import java.util.Map;
import com.landray.kmss.util.SpringBeanUtil;
import java.util.HashMap;
import java.util.Arrays;
import java.util.ArrayList;
import com.landray.kmss.util.StringUtil;
import java.util.Collections;
import com.landray.kmss.framework.service.plugin.Plugin;
import java.util.Comparator;
import com.landray.kmss.framework.plugin.core.config.IExtension;
import com.landray.kmss.sys.attachment.watermark.service.IYuFeiWatermarkService;
import java.util.List;
import com.landray.kmss.sys.attachment.watermark.service.IWatermarkService;
import org.slf4j.LoggerFactory;
import org.slf4j.Logger;

public class WatermarkPlugin
{
    private static final Logger logger;
    public static final String EX_ITEM_NAME_SERVICE = "service";
    public static final String YF_ITEM_NAME_SERVICE = "yfService";
    public static final String P_PN = "providerName";
    public static final String P_BEAN = "bean";
    public static final String P_CJSP = "configJsp";
    public static final String P_ORDER = "order";
    public static final String EXTENSION_WATERMARK_ADDWATERMARK_SERVICE;
    
    static {
        logger = LoggerFactory.getLogger(WatermarkPlugin.class);
        EXTENSION_WATERMARK_ADDWATERMARK_SERVICE = IWatermarkService.class.getPackage().getName();
    }
    
    private WatermarkPlugin() {
    }
    
    public static List<IWatermarkService> getWatermarkServices() {
        return getService(IWatermarkService.class, null, null, "service");
    }
    
    public static IWatermarkService getWatermarkService(final String providerName) {
        final List<IWatermarkService> services = getService(IWatermarkService.class, null, providerName, "service");
        if (!services.isEmpty()) {
            return services.get(0);
        }
        return null;
    }
    
    public static List<IYuFeiWatermarkService> getYuFeiWatermarkServices() {
        return getService(IYuFeiWatermarkService.class, null, null, "yfService");
    }
    
    public static IYuFeiWatermarkService getYuFeiWatermarkService(final String providerName) {
        final List<IYuFeiWatermarkService> services = getService(IYuFeiWatermarkService.class, null, providerName, "yfService");
        if (!services.isEmpty()) {
            return services.get(0);
        }
        return null;
    }
    
    private static void sortByOrder(final List<IExtension> enabledExtensions) {
        Collections.sort(enabledExtensions, new Comparator<IExtension>() {
            @Override
            public int compare(final IExtension o1, final IExtension o2) {
                final Object obj1 = Plugin.getParamValue(o1, "order");
                final Object obj2 = Plugin.getParamValue(o2, "order");
                int order1;
                int order2;
                if (obj1 instanceof Integer) {
                    order1 = (int)obj1;
                    order2 = (int)obj2;
                }
                else {
                    order1 = Integer.valueOf((String)obj1);
                    order2 = Integer.valueOf((String)obj2);
                }
                return order1 - order2;
            }
        });
    }
    
    private static <T> List<T> getService(final Class<T> type, String model, final String providerName, final String itemName) {
        if (StringUtil.isNull(model)) {
            model = "*";
        }
        final List<T> rtn = new ArrayList<T>();
        final IExtension[] extensions = Plugin.getExtensions(type.getPackage().getName(), model, itemName);
        if (extensions == null || extensions.length < 1) {
            return rtn;
        }
        final List<IExtension> enabledExtensions = Arrays.asList(extensions);
        Object bean = null;
        if (StringUtil.isNull(providerName)) {
            sortByOrder(enabledExtensions);
            final Map<String, String> cache = new HashMap<String, String>();
            for (final IExtension extesion : enabledExtensions) {
                final String beanName = Plugin.getParamValueString(extesion, "bean");
                final String pn = Plugin.getParamValueString(extesion, "providerName");
                if (cache.containsKey(pn)) {
                    if (!WatermarkPlugin.logger.isDebugEnabled()) {
                        continue;
                    }
                    WatermarkPlugin.logger.warn("There is a extension with provider {name:" + pn + ", bean: " + cache.get(pn) + "}, bean: " + beanName + " will be ignored.");
                }
                else {
                    cache.put(pn, beanName);
                    bean = SpringBeanUtil.getBean(beanName);
                    if (!type.isInstance(bean)) {
                        throw new RuntimeException("the bean:" + beanName + " must implements the interface:" + type.getName());
                    }
                    rtn.add(type.cast(bean));
                }
            }
        }
        else {
            for (final IExtension extesion2 : enabledExtensions) {
                final String pn2 = Plugin.getParamValueString(extesion2, "providerName");
                if (providerName.equalsIgnoreCase(pn2)) {
                    final String beanName = Plugin.getParamValueString(extesion2, "bean");
                    bean = SpringBeanUtil.getBean(beanName);
                    if (!type.isInstance(bean)) {
                        throw new RuntimeException("the bean:" + beanName + " must implements the interface:" + type.getName());
                    }
                    rtn.add(type.cast(bean));
                    break;
                }
            }
        }
        return rtn;
    }
}
